;*************************************************************
;; PROGRAM: Joyinp.bb
;; Author: Krylar
;; Last Updated: 10/14/2000
;;
;; Rotating an image using the joystick
;*************************************************************


Const iScreenWidth=800               ;what's our screen width?
Const iScreenHeight=600              ;what's our screen height?
Const iNumRotations=36               ;how many rotations do we want?

Dim imgShipFrames(iNumRotations)     ;Holds the rotated images

; Initialize BB's Graphics using our constants
Graphics iScreenWidth, iScreenHeight

; Set up our primary drawing surface to be the BackBuffer
; this is to allow for page-flip animation
SetBuffer BackBuffer()

; load up our graphics
LoadGraphics()

;What is the current frame being displayed?
Global iCurrentFrame = 0             

; find the x,y coordinates to position the ship in the center
; of the screen
Global iShipX = (iScreenWidth/2) - (ImageWidth(imgShipFrames(0)) / 2)
Global iShipY = (iScreenHeight/2) - (ImageHeight(imgShipFrames(0)) / 2)

; loop until the player hits the ESC key
While Not KeyHit(1)

  ; check to see if the Joystick is moving
  CheckJoystick()

  ; clear the BackBuffer
  Cls

  ; draw the current Ship Frame on the center of the BackBuffer
  DrawImage(imgShipFrames(iCurrentFrame),iShipX,iShipY)

  ; Flip the BackBuffer to the Display
  Flip

Wend
  
; end the program, return control to Windows
End


;*************************************************************
; FUNCTION: CheckJoystick()
; Author: Krylar
; Last Updated: 10/14/2000
;
; This function checks to see if the joystick has been moved 
; To the Left Or Right...does Not check up/down
;*************************************************************
Function CheckJoystick()

 ; call the JoyXDir() function to see if
 ; the stick is being pushed left
 If JoyXDir() < 0 Then
    ; change the current frame (spin it to the left)
    iCurrentFrame = iCurrentFrame - 1

    ; if the current frame counter is less than 0
    ; reset it to the number of rotations - 1
    If iCurrentFrame < 0 
       iCurrentFrame = iNumRotations - 1
    EndIf
 EndIf

 ; call the JoyXDir() function to see if
 ; the stick is being pushed right
 If JoyXDir() > 0 Then
    ; change the current frame (spin it to the right)
    iCurrentFrame = iCurrentFrame + 1

    ; if the current frame counter goes past our number of
    ; allowable rotations, reset it to 0
    If iCurrentFrame > iNumRotations - 1
       iCurrentFrame = 0
    EndIf
 EndIf

End Function


;*************************************************************
; FUNCTION: LoadGraphics()
; Author: Krylar
; Last Updated: 10/14/2000
;
; This function loads up the ship image and builds a rotated 
; version of it
;*************************************************************
Function LoadGraphics()

 ; tell BB to handle the centering of our images
 AutoMidHandle True

 ; Load the player image and point to it with "imgTemp"
 imgTemp = LoadImage("graphics\player.bmp")

 ; See if the image was loaded successfully (the "imgTemp" will be 0 if not)
 If imgTemp = 0 Then
    Text 100,100,"Invalid Image!"    ; tell the user it was invalid if 0
 Else
    ; set it's mask (transparent color)
    MaskImage imgTemp,0,0,0

    ; now run through the loop and rotate the image
    For iLoop=0 To iNumRotations-1

        ; first let's copy the original image into the current frame
   	    imgShipFrames(iLoop)=CopyImage( imgTemp )

        ; then rotate that frame the appropriate number of degrees
	    RotateImage imgShipFrames(iLoop),iLoop*360/iNumRotations
    Next
 EndIf

End Function


