;***************************************************************************
;*        Title: 2D Space Game Physics
;*     Coded By: Krylar
;* Last Updated: 09/30/2000
;***************************************************************************

; Set up the constant vars
Const g_ScreenWidth=800, g_ScreenHeight=600
Const iNumRotations=36

; Setup Keyboard Constants (Arrows, KeyPad Arrows)
Const LeftArrow=203, RightArrow=205
Const UpArrow=200,   DownArrow=208
Const LeftKPArrow=75, RightKPArrow=77
Const UpKPArrow=72, DownKPArrow=80

; Initialize the graphics
Graphics g_ScreenWidth, g_ScreenHeight

; Setup the Backbuffer for page flipping
SetBuffer BackBuffer()

; Setup our Globals
Global Image_StarsFar                            ;Holds the backround stars image
Global Image_StarsMid                            ;Holds the backround stars image
Global Image_StarsClose                          ;Holds the backround stars image
Global Scroll_StarsX#, Scroll_StarsY#            ;the back parallaxing vars
Global Scroll_StarsX2#, Scroll_StarsY2#          ;the middle parallaxing vars
Global Scroll_StarsX3#, Scroll_StarsY3#          ;the front parallaxing vars
Global iSpeedModifier#							 ;holds top speed per x,y angle

; Image holders
Dim Ship_Player(36)                           ;Holds the Frieghter images

; Setup our physics tables
Dim xSinTable#(iNumRotations) 
Dim yCosTable#(iNumRotations)
Dim xTopSpeedTable#(iNumRotations) 
Dim yTopSpeedTable#(iNumRotations)
Dim xTSTable#(iNumRotations/4)
Dim yTSTable#(iNumRotations/4)

; build our Ship structure
Type Ship
	Field iShipDir
	Field iX,iY
	Field dX#,dY#
	Field dThrust#, dTopSpeed#
	Field dBraking#
End Type

; Make an instance of the Ship structure called "Player"
Global Player.Ship = New Ship
Player\iX=400
Player\iY=300
Player\dX=0
Player\dY=0
Player\iShipDir=0
Player\dThrust=.005
Player\dTopSpeed=2
Player\dBraking=.975

; Call the functions that setup our 2D "Physics"
PrecomputeSinCosTables()
PrecomputeTopSpeedTables()

; Load the graphics from the file(s)
LoadGraphics()

; initialize the timer
game_state=0
game_timer=0
time=MilliSecs()


;*******************************************************************
;* 
;* main loop...keep looping until the user hits ESC
;* 
;*******************************************************************
While Not KeyDown(1)

 n_time=MilliSecs()
 elapsed=n_time-time
 If elapsed>25		;slowing down! - clamp update to 40 FPS (1000/40=25 millisecs)
 	cnt=elapsed/25
 	For k=1 To cnt
		UpdatePlayer()
	Next
	time=time+cnt*25
 Else				;leave frame synced...
	UpdatePlayer()
	time=n_time
 EndIf

 ; Build out the Starfield based on the movement of the ship
 RenderStarfield()

 ; Draw the Player
 RenderPlayer()

 Text 0,10,"dX = "+Player\dX,0,1
 Text 0,30,"dY = "+Player\dY,0,1
 Text 0,50,"Dir = "+Player\iShipDir,0,1


 ; Flip the pages
 Flip

 ; Clear the screen
 ClsColor 0,0,0

Wend
 
; Delete the Player instance
Delete Player

; End of the program
End


;*******************************************************************
;* Update the game...check the keyboard and such
;*******************************************************************
Function UpdatePlayer()

 ; START the RightArrow check
 If KeyDown(RightArrow) Or KeyDown(RightKPArrow) Then
    ; spin the ship to the right
    Player\iShipDir = Player\iShipDir + 1
    If(Player\iShipDir > iNumRotations-1) Then 
       Player\iShipDir = 0
    EndIf
 EndIf
 ; END the RightArrow check


 ; START the LeftArrow check
 If KeyDown(LeftArrow) Or KeyDown(LeftKPArrow) Then
    ; spin the ship to the left
    Player\iShipDir = Player\iShipDir - 1
    If(Player\iShipDir < 0) Then 
       Player\iShipDir = iNumRotations-1
    EndIf
 EndIf
 ; END the LeftArrow check


 ; START the UpArrow check
 ; This section controls the Thrust of the ship and so on
 ; study this one carefully
 If KeyDown(UpArrow) Or KeyDown(UpKPArrow) Then
    ; add the thrust information to the current x,y vars
    ; of the ship's directional vectors
    Player\dX# = Player\dX# + (xSinTable#(Player\iShipDir) * Player\dThrust#)
    Player\dY# = Player\dY# + (yCosTable#(Player\iShipDir) * Player\dThrust#)


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;; This handles the N, N/E quadrant
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    If Player\iShipDir < 9 Then

       ;Handling the X axis there
       If Player\dX# < 0 - xTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the X axis due to angle
          If Player\iShipDir < 5
             ; use the 5th xSinTable spot to decrease X speed at a reasonable rate
             Player\dX# = Player\dX# - (xSinTable#(5) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Sin information for normal decrease
             Player\dX# = Player\dX# - (xSinTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf

       ;Handling the Y axis there
       If Player\dY# > yTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the Y axis due to angle
          If Player\iShipDir > 4
             ; use the 5th xSinTable spot to decrease Y speed at a reasonable rate
             Player\dY# = Player\dY# - (yCosTable#(5) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Cos information for normal decrease
             Player\dY# = Player\dY# - (yCosTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf
    EndIf


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;; This handles the E, S/E quadrant
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    If Player\iShipDir >= 9 And Player\iShipDir < 18 Then

       ;Handling the X axis there
       If Player\dX# < 0 - xTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the X axis due to angle
          If Player\iShipDir > 13
             ; use the 14th xSinTable spot to decrease X speed at a reasonable rate
             Player\dX# = Player\dX# - (xSinTable#(14) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Sin information for normal decrease
             Player\dX# = Player\dX# - (xSinTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf

       ;Handling the Y axis there
       If Player\dY# < 0 - yTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the Y axis due to angle
          If Player\iShipDir < 14
             ; use the 14th xSinTable spot to decrease Y speed at a reasonable rate
             Player\dY# = Player\dY# - (yCosTable#(14) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Cos information for normal decrease
             Player\dY# = Player\dY# - (yCosTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf

    EndIf


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;; This handles the S, S/W quadrant
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    If Player\iShipDir >= 18 And Player\iShipDir < 27 Then

       ;Handling the X axis there
       If Player\dX# > xTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the X axis due to angle
          If Player\iShipDir < 23
             ; use the 23rd xSinTable spot to decrease X speed at a reasonable rate
             Player\dX# = Player\dX# - (xSinTable#(23) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Sin information for normal decrease
             Player\dX# = Player\dX# - (xSinTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf

       ;Handling the Y axis there
       If Player\dY# < 0 - yTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the Y axis due to angle
          If Player\iShipDir > 22
             ; use the 23rd xSinTable spot to decrease Y speed at a reasonable rate
             Player\dY# = Player\dY# - (yCosTable#(23) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Cos information for normal decrease
             Player\dY# = Player\dY# - (yCosTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf
    EndIf


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;; This handles the W, N/W quadrant
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    If Player\iShipDir >= 27 Then

       ;Handling the X axis there
       If Player\dX# > xTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the X axis due to angle
          If Player\iShipDir > 31
             ; use the 32nd xSinTable spot to decrease X speed at a reasonable rate
             Player\dX# = Player\dX# - (xSinTable#(32) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Sin information for normal decrease
             Player\dX# = Player\dX# - (xSinTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf

       ;Handling the Y axis there
       If Player\dY# > yTopSpeedTable#(Player\iShipDir) Then
          ; see if we need to subtract faster off the Y axis due to angle
          If Player\iShipDir < 32
             ; use the 32nd xSinTable spot to decrease Y speed at a reasonable rate
             Player\dY# = Player\dY# - (yCosTable#(32) * Player\dThrust#)
          Else
             ; Otherwise just use the normal Cos information for normal decrease
             Player\dY# = Player\dY# - (yCosTable#(Player\iShipDir) * Player\dThrust#)
          EndIf
       EndIf
    EndIf
 EndIf
 ; END the UpArrow check


 ; START the DownArrow check
 If KeyDown(DownArrow) Or KeyDown(DownKPArrow) Then
	; slow down the ship
    Player\dX = Player\dX * Player\dBraking
	Player\dY = Player\dY * Player\dBraking

	; check for stop on ship's X axis
	If Player\dX < .001 And Player\dX > -.001 Then
		Player\dX = 0
	EndIf
	
	; check for stop on ship's Y axis
	If Player\dY < .001 And Player\dY > -.001 Then
		Player\dY = 0
	EndIf
 EndIf
 ; END the DownArrow check

End Function



;*******************************************************************
;* Render the Starfield
;*******************************************************************
Function RenderStarfield()
    ; Fill the whole background with the 3 star layers
    ; in their current positions
	TileBlock Image_StarsFar,Scroll_StarsX,Scroll_StarsY
	TileImage Image_StarsMid,Scroll_StarsX2*2,Scroll_StarsY2*2
	TileImage Image_StarsClose,Scroll_StarsX3*3,Scroll_StarsY3*3
	
	; setup the 3 layers of data for the next rendering
	Scroll_StarsX = Scroll_StarsX + Player\dX
	Scroll_StarsY = Scroll_StarsY + Player\dY
	Scroll_StarsX2 = Scroll_StarsX + 7 + Player\dX
	Scroll_StarsY2 = Scroll_StarsY + 7 + Player\dY
	Scroll_StarsX3 = Scroll_StarsX + 23 + Player\dX
	Scroll_StarsY3 = Scroll_StarsY + 23 + Player\dY
End Function


;*******************************************************************
;* Render the Player
;*******************************************************************
Function RenderPlayer()
 ; display the ship's current frame in the center of the screen
 DrawImage Ship_Player(Player\iShipDir),Player\iX,Player\iY
End Function


;*******************************************************************
;* Load the graphics
;*******************************************************************
Function LoadGraphics()
 Local Image_Temp   ; pointer to a Temp image

 ; be lazy and let BB handle the centering of the images
 ; during rotation
 AutoMidHandle True

 ; load up the star backgrounds (3 total)
 Image_StarsFar=LoadImage( "graphics\starsfar.bmp" )
 Image_StarsMid=LoadImage( "graphics\starsmid.bmp" )
 Image_StarsClose=LoadImage( "graphics\stars.bmp" )

 ; load up our ship image
 Image_Temp=LoadImage( "graphics\ship1.bmp" )

 ; set it's mask (transparent color)
 MaskImage Image_Temp,0,0,0

 ; now run through the loop and rotate the image
 ; around at 10 degree increments.  
 For iLoop=0 To iNumRotations-1
 	 Ship_Player(iLoop)=CopyImage( Image_Temp )
	 RotateImage Ship_Player(iLoop),iLoop*360/iNumRotations
 Next

End Function


;*******************************************************************
;* Function to Precompute the Sin and Cos tables
;*******************************************************************
Function PrecomputeSinCosTables()
 Local iAngle# = 0

 ; run through the full rotation cycle, and use Sin and Cos
 ; at 10-degree increments
 For iAngle = 0 To iNumRotations-1
     xSinTable#(iAngle) = -Sin(iAngle*10)
     yCosTable#(iAngle) = Cos(iAngle*10)
 Next
End Function


;*******************************************************************
;* Function to Precompute Top Speed Tables
;*******************************************************************
Function PrecomputeTopSpeedTables()
 Local iSpeedControl#
 Local iXAngle = 9
 Local iYAngle = 0

 ; setup our speed modifier
 iSpeedModifier# = (Player\dTopSpeed# / (iNumRotations/4))
 ; save our Top Speed
 iSpeedControl# = Player\dTopSpeed#

 For iAngle = 0 To iNumRotations / 4
     ; assign the speed control per x,y angle
     xTSTable#(iXAngle) = iSpeedControl#
     yTSTable#(iYAngle) = iSpeedControl#

     ; make sure that arrays are being loaded 
     ; in opposite directions
     iXAngle = iXAngle - 1
     iYAngle = iYAngle + 1

     ; get that ABS value of our current speed control
     ; minus the speed modifier
     iSpeedControl# = Abs(iSpeedControl# - iSpeedModifier#)

     ; if it's below a certain level, just set it to zero
     If iSpeedControl# < .0001 Then
        iSpeedControl# = .000000
     EndIf
 Next

 ; Now we're going to load up the angle top speed tables
 ; so we can use the iShipDir to get the appropriate
 ; speed value per x,y angle when needed without having
 ; to do wacky math real-time
 For iAngle = 0 To 8
     xTopSpeedTable#(iAngle) = xTSTable#(iAngle)
     yTopSpeedTable#(iAngle) = yTSTable#(iAngle)
 Next

 iNewAngle = 9
 For iAngle = 9 To 17
     xTopSpeedTable#(iAngle) = xTSTable#(iNewAngle)
     yTopSpeedTable#(iAngle) = yTSTable#(iNewAngle)
     iNewAngle = iNewAngle - 1
 Next

 iNewAngle = 0
 For iAngle = 18 To 26
     xTopSpeedTable#(iAngle) = xTSTable#(iNewAngle)
     yTopSpeedTable#(iAngle) = yTSTable#(iNewAngle)
     iNewAngle = iNewAngle + 1
 Next

 iNewAngle = 9
 For iAngle = 27 To 35
     xTopSpeedTable#(iAngle) = xTSTable#(iNewAngle)
     yTopSpeedTable#(iAngle) = yTSTable#(iNewAngle)
     iNewAngle = iNewAngle - 1
 Next
End Function